/* pmmemtobutton.c                                                  */
/*                                                                  */
/* This demonstrates how to draw into an image.  This example uses  */
/* the PmMem*() functions to draw into a memory context.  When      */
/* finished drawing, the memory context is then dumped into an      */
/* image.  The image is then used as the image displayed on a       */
/* button.                                                          */
/*                                                                  */
/* To compile, you must link with phrender_l.lib.  For example:     */
/*                                                                  */
/*  cc -w3 -opmmemtobutton -lphrender_s -lphoton_s pmmemtobutton.c  */
/*                                                                  */

#ifdef __USAGE
%C - uses a memory context to create an image for a button widget

%C
#endif

#include <mem.h>
#include <photon/PhRender.h>
#include <Pt.h>

main( int argc, char *argv[] )
{
    PhArea_t    area = { 80, 20, 80, 40 };
    PhDim_t     dim = { 240, 80 };
    PhImage_t   image;
    PtArg_t     arg[3];
    PtWidget_t  *button, *window;
    
    PtSetArg( &arg[0], Pt_ARG_WINDOW_TITLE, "Memory Context Sample", 0 );
    PtSetArg( &arg[1], Pt_ARG_DIM, &dim, 0 );
    window = PtAppInit( NULL, &argc, argv, 2, arg );

    create_image( &image, &area.size );

    PtSetArg( &arg[0], Pt_ARG_LABEL_TYPE, Pt_IMAGE, 0 );
    PtSetArg( &arg[1], Pt_ARG_AREA, &area, 0 );
    PtSetArg( &arg[2], Pt_ARG_LABEL_DATA, &image, sizeof(PhImage_t) );
    button = PtCreateWidget( PtButton, NULL, 3, arg );
    
    PtRealizeWidget( window );
    PtMainLoop();
}

void
create_image( PhImage_t *image, PhDim_t *dim )
{
    PhPoint_t           translation = { 0, 0 }, center, radii;
    PmMemoryContext_t   *mc; 
    short               bytes_per_pixel = 3;
    
    memset( image, 0, sizeof(PhImage_t) );
    image->type = Pg_IMAGE_DIRECT_888; // 3 bytes per pixel with this type
    image->size = *dim;
    image->image = PgShmemCreate( dim->w * dim->h * bytes_per_pixel, NULL );
    
    mc = PmMemCreateMC( image, dim, &translation );
    
    PmMemStart( mc );
    // now all drawing goes into the memory context
    
    // draw whatever we want to appear in the image
    center.x = dim->w / 2;
    center.y = dim->h / 2;
    radii = center;
    PgSetFillColor( Pg_WHITE );
    PgSetStrokeColor( Pg_RED );
    PgDrawEllipse( &center, &radii, Pg_DRAW_FILL_STROKE );
    PgSetStrokeColor( Pg_GREEN );
    PgDrawILine( 0, 0, dim->w-1, dim->h-1 );
    
    PmMemFlush( mc, image ); // get the image
    PmMemStop( mc );
    // now all drawing goes to the default drawing context
    
    PmMemReleaseMC( mc );
}
